from typing import Any, Dict, Optional, Union

import httpx

from ...client import AuthenticatedClient
from ...models.available_pairs import AvailablePairs
from ...models.candle_type import CandleType
from ...models.http_validation_error import HTTPValidationError
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    client: AuthenticatedClient,
    timeframe: Union[Unset, None, str] = UNSET,
    stake_currency: Union[Unset, None, str] = UNSET,
    candletype: Union[Unset, None, CandleType] = UNSET,
) -> Dict[str, Any]:
    url = "{}/api/v1/available_pairs".format(client.base_url)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {}
    params["timeframe"] = timeframe

    params["stake_currency"] = stake_currency

    json_candletype: Union[Unset, None, str] = UNSET
    if not isinstance(candletype, Unset):
        json_candletype = candletype.value if candletype else None

    params["candletype"] = json_candletype

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "get",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(*, response: httpx.Response) -> Optional[Union[AvailablePairs, HTTPValidationError]]:
    if response.status_code == 200:
        response_200 = AvailablePairs.from_dict(response.json())

        return response_200
    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422
    return None


def _build_response(*, response: httpx.Response) -> Response[Union[AvailablePairs, HTTPValidationError]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    timeframe: Union[Unset, None, str] = UNSET,
    stake_currency: Union[Unset, None, str] = UNSET,
    candletype: Union[Unset, None, CandleType] = UNSET,
) -> Response[Union[AvailablePairs, HTTPValidationError]]:
    """List Available Pairs

    Args:
        timeframe (Union[Unset, None, str]):
        stake_currency (Union[Unset, None, str]):
        candletype (Union[Unset, None, CandleType]): Enum to distinguish candle types

    Returns:
        Response[Union[AvailablePairs, HTTPValidationError]]
    """

    kwargs = _get_kwargs(
        client=client,
        timeframe=timeframe,
        stake_currency=stake_currency,
        candletype=candletype,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: AuthenticatedClient,
    timeframe: Union[Unset, None, str] = UNSET,
    stake_currency: Union[Unset, None, str] = UNSET,
    candletype: Union[Unset, None, CandleType] = UNSET,
) -> Optional[Union[AvailablePairs, HTTPValidationError]]:
    """List Available Pairs

    Args:
        timeframe (Union[Unset, None, str]):
        stake_currency (Union[Unset, None, str]):
        candletype (Union[Unset, None, CandleType]): Enum to distinguish candle types

    Returns:
        Response[Union[AvailablePairs, HTTPValidationError]]
    """

    return sync_detailed(
        client=client,
        timeframe=timeframe,
        stake_currency=stake_currency,
        candletype=candletype,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    timeframe: Union[Unset, None, str] = UNSET,
    stake_currency: Union[Unset, None, str] = UNSET,
    candletype: Union[Unset, None, CandleType] = UNSET,
) -> Response[Union[AvailablePairs, HTTPValidationError]]:
    """List Available Pairs

    Args:
        timeframe (Union[Unset, None, str]):
        stake_currency (Union[Unset, None, str]):
        candletype (Union[Unset, None, CandleType]): Enum to distinguish candle types

    Returns:
        Response[Union[AvailablePairs, HTTPValidationError]]
    """

    kwargs = _get_kwargs(
        client=client,
        timeframe=timeframe,
        stake_currency=stake_currency,
        candletype=candletype,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    timeframe: Union[Unset, None, str] = UNSET,
    stake_currency: Union[Unset, None, str] = UNSET,
    candletype: Union[Unset, None, CandleType] = UNSET,
) -> Optional[Union[AvailablePairs, HTTPValidationError]]:
    """List Available Pairs

    Args:
        timeframe (Union[Unset, None, str]):
        stake_currency (Union[Unset, None, str]):
        candletype (Union[Unset, None, CandleType]): Enum to distinguish candle types

    Returns:
        Response[Union[AvailablePairs, HTTPValidationError]]
    """

    return (
        await asyncio_detailed(
            client=client,
            timeframe=timeframe,
            stake_currency=stake_currency,
            candletype=candletype,
        )
    ).parsed
